# coding=utf-8
# Copyright 2015 Pants project contributors (see CONTRIBUTORS.md).
# Licensed under the Apache License, Version 2.0 (see LICENSE).

from __future__ import (absolute_import, division, generators, nested_scopes, print_function,
                        unicode_literals, with_statement)

from pants.goal.task_registrar import TaskRegistrar as task

from pants.contrib.scrooge.tasks.scrooge_gen import ScroogeGen
from pants.contrib.scrooge.tasks.thrift_linter import ThriftLinter


def register_goals():
  task(name='thrift-linter', action=ThriftLinter).install()
  task(name='scrooge', action=ScroogeGen).install('gen')



# coding=utf-8
# Copyright 2014 Pants project contributors (see CONTRIBUTORS.md).
# Licensed under the Apache License, Version 2.0 (see LICENSE).

from __future__ import (absolute_import, division, generators, nested_scopes, print_function,
                        unicode_literals, with_statement)

import os
import re
import tempfile
from collections import defaultdict, namedtuple

from pants.backend.codegen.thrift.java.java_thrift_library import JavaThriftLibrary
from pants.backend.codegen.thrift.java.thrift_defaults import ThriftDefaults
from pants.backend.jvm.tasks.nailgun_task import NailgunTask
from pants.base.exceptions import TargetDefinitionException, TaskError
from pants.build_graph.address import Address
from pants.build_graph.address_lookup_error import AddressLookupError
from pants.task.simple_codegen_task import SimpleCodegenTask
from pants.util.dirutil import safe_mkdir, safe_open
from pants.util.memo import memoized_method, memoized_property
from twitter.common.collections import OrderedSet

from pants.contrib.scrooge.tasks.java_thrift_library_fingerprint_strategy import \
  JavaThriftLibraryFingerprintStrategy
from pants.contrib.scrooge.tasks.thrift_util import calculate_compile_sources


_RPC_STYLES = frozenset(['sync', 'finagle', 'ostrich'])


class ScroogeGen(SimpleCodegenTask, NailgunTask):

  DepInfo = namedtuple('DepInfo', ['service', 'structs'])
  PartialCmd = namedtuple('PartialCmd',
    ['language', 'rpc_style', 'namespace_map', 'default_java_namespace', 'include_paths'])

  @classmethod
  def register_options(cls, register):
    super(ScroogeGen, cls).register_options(register)
    register('--verbose', type=bool, help='Emit verbose output.')
    register('--strict', fingerprint=True, type=bool,
             help='Enable strict compilation.')
    register('--service-deps', default={}, advanced=True, type=dict,
             help='A map of language to targets to add as dependencies of '
                  'synthetic thrift libraries that contain services.')
    register('--structs-deps', default={}, advanced=True, type=dict,
             help='A map of language to targets to add as dependencies of '
                  'synthetic thrift libraries that contain structs.')
    register('--target-types',
             default={'scala': 'scala_library', 'java': 'java_library', 'android': 'java_library'},
             advanced=True,
             type=dict,
             help='Registered target types.')
    cls.register_jvm_tool(register, 'scrooge-gen')

  @classmethod
  def subsystem_dependencies(cls):
    return super(ScroogeGen, cls).subsystem_dependencies() + (ThriftDefaults,)

  @classmethod
  def product_types(cls):
    return ['java', 'scala']

  @classmethod
  def implementation_version(cls):
    return super(ScroogeGen, cls).implementation_version() + [('ScroogeGen', 3)]

  @classmethod
  def get_fingerprint_strategy(cls):
    return JavaThriftLibraryFingerprintStrategy(ThriftDefaults.global_instance())

  def __init__(self, *args, **kwargs):
    super(ScroogeGen, self).__init__(*args, **kwargs)
    self._thrift_defaults = ThriftDefaults.global_instance()
    self._depinfo = None

  # TODO(benjy): Use regular os-located tmpfiles, as we do everywhere else.
  def _tempname(self):
    # don't assume the user's cwd is buildroot
    pants_workdir = self.get_options().pants_workdir
    tmp_dir = os.path.join(pants_workdir, 'tmp')
    safe_mkdir(tmp_dir)
    fd, path = tempfile.mkstemp(dir=tmp_dir, prefix='')
    os.close(fd)
    return path

  def _resolve_deps(self, depmap):
    """Given a map of gen-key=>target specs, resolves the target specs into references."""
    deps = defaultdict(lambda: OrderedSet())
    for category, depspecs in depmap.items():
      dependencies = deps[category]
      for depspec in depspecs:
        dep_address = Address.parse(depspec)
        try:
          self.context.build_graph.maybe_inject_address_closure(dep_address)
          dependencies.add(self.context.build_graph.get_target(dep_address))
        except AddressLookupError as e:
          raise AddressLookupError('{}\n  referenced from {} scope'.format(e, self.options_scope))
    return deps

  def _validate_language(self, target):
    language = self._thrift_defaults.language(target)
    if language not in self._registered_language_aliases():
      raise TargetDefinitionException(
          target,
          'language {} not supported: expected one of {}.'.format(language, self._registered_language_aliases().keys()))
    return language

  def _validate_rpc_style(self, target):
    rpc_style = self._thrift_defaults.rpc_style(target)
    if rpc_style not in _RPC_STYLES:
      raise TargetDefinitionException(
          target,
          'rpc_style {} not supported: expected one of {}.'.format(rpc_style, _RPC_STYLES))
    return rpc_style

  @memoized_method
  def _registered_language_aliases(self):
    return self.get_options().target_types

  @memoized_method
  def _target_type_for_language(self, language):
    alias_for_lang = self._registered_language_aliases()[language]
    registered_aliases = self.context.build_file_parser.registered_aliases()
    target_types = registered_aliases.target_types_by_alias.get(alias_for_lang, None)
    if not target_types:
      raise TaskError('Registered target type `{0}` for language `{1}` does not exist!'.format(alias_for_lang, language))
    if len(target_types) > 1:
      raise TaskError('More than one target type registered for language `{0}`'.format(language))
    return next(iter(target_types))

  def execute_codegen(self, target, target_workdir):
    self._validate_compiler_configs([target])
    self._must_have_sources(target)

    namespace_map = self._thrift_defaults.namespace_map(target)

    partial_cmd = self.PartialCmd(
        language=self._validate_language(target),
        rpc_style=self._validate_rpc_style(target),
        namespace_map=tuple(sorted(namespace_map.items())) if namespace_map else (),
        default_java_namespace=self._thrift_defaults.default_java_namespace(target),
        include_paths=target.include_paths)

    self.gen(partial_cmd, target, target_workdir)

  def gen(self, partial_cmd, target, target_workdir):
    import_paths, _ = calculate_compile_sources([target], self.is_gentarget)

    args = []

    if partial_cmd.default_java_namespace:
      args.extend(['--default-java-namespace', partial_cmd.default_java_namespace])

    for import_path in import_paths:
      args.extend(['--import-path', import_path])

    args.extend(['--language', partial_cmd.language])

    for lhs, rhs in partial_cmd.namespace_map:
      args.extend(['--namespace-map', '%s=%s' % (lhs, rhs)])

    if partial_cmd.rpc_style == 'ostrich':
      args.append('--finagle')
      args.append('--ostrich')
    elif partial_cmd.rpc_style == 'finagle':
      args.append('--finagle')

    args.extend(['--dest', target_workdir])

    if not self.get_options().strict:
      args.append('--disable-strict')

    if partial_cmd.include_paths:
      for include_path in partial_cmd.include_paths:
        args.extend(['--include-path', include_path])

    if self.get_options().verbose:
      args.append('--verbose')

    gen_file_map_path = os.path.relpath(self._tempname())
    args.extend(['--gen-file-map', gen_file_map_path])

    args.extend(target.sources_relative_to_buildroot())

    classpath = self.tool_classpath('scrooge-gen')
    jvm_options = list(self.get_options().jvm_options)
    jvm_options.append('-Dfile.encoding=UTF-8')
    returncode = self.runjava(classpath=classpath,
                              main='com.twitter.scrooge.Main',
                              jvm_options=jvm_options,
                              args=args,
                              workunit_name='scrooge-gen')
    if 0 != returncode:
      raise TaskError('Scrooge compiler exited non-zero for {} ({})'.format(target, returncode))

  SERVICE_PARSER = re.compile(r'^\s*service\s+(?:[^\s{]+)')

  def _declares_service(self, source):
    with open(source) as thrift:
      return any(line for line in thrift if self.SERVICE_PARSER.search(line))

  def parse_gen_file_map(self, gen_file_map_path, outdir):
    d = defaultdict(set)
    with safe_open(gen_file_map_path, 'r') as deps:
      for dep in deps:
        src, cls = dep.strip().split('->')
        src = os.path.relpath(src.strip())
        cls = os.path.relpath(cls.strip(), outdir)
        d[src].add(cls)
    return d

  def is_gentarget(self, target):
    if not isinstance(target, JavaThriftLibrary):
      return False

    # We only handle requests for 'scrooge' compilation and not, for example 'thrift', aka the
    # Apache thrift compiler
    return self._thrift_defaults.compiler(target) == 'scrooge'

  def _validate_compiler_configs(self, targets):
    assert len(targets) == 1, ("TODO: This method now only ever receives one target. Simplify.")
    ValidateCompilerConfig = namedtuple('ValidateCompilerConfig', ['language', 'rpc_style'])

    def compiler_config(tgt):
      # Note compiler is not present in this signature. At this time
      # Scrooge and the Apache thrift generators produce identical
      # java sources, and the Apache generator does not produce scala
      # sources. As there's no permutation allowing the creation of
      # incompatible sources with the same language+rpc_style we omit
      # the compiler from the signature at this time.
      return ValidateCompilerConfig(language=self._thrift_defaults.language(tgt),
                                    rpc_style=self._thrift_defaults.rpc_style(tgt))

    mismatched_compiler_configs = defaultdict(set)

    for target in filter(lambda t: isinstance(t, JavaThriftLibrary), targets):
      mycompilerconfig = compiler_config(target)

      def collect(dep):
        if mycompilerconfig != compiler_config(dep):
          mismatched_compiler_configs[target].add(dep)
      target.walk(collect, predicate=lambda t: isinstance(t, JavaThriftLibrary))

    if mismatched_compiler_configs:
      msg = ['Thrift dependency trees must be generated with a uniform compiler configuration.\n\n']
      for tgt in sorted(mismatched_compiler_configs.keys()):
        msg.append('%s - %s\n' % (tgt, compiler_config(tgt)))
        for dep in mismatched_compiler_configs[tgt]:
          msg.append('    %s - %s\n' % (dep, compiler_config(dep)))
      raise TaskError(''.join(msg))

  def _must_have_sources(self, target):
    if isinstance(target, JavaThriftLibrary) and not target.payload.sources.source_paths:
      raise TargetDefinitionException(target, 'no thrift files found')

  def synthetic_target_type(self, target):
    language = self._thrift_defaults.language(target)
    return self._target_type_for_language(language)

  def synthetic_target_extra_dependencies(self, target, target_workdir):
    deps = OrderedSet(self._thrift_dependencies_for_target(target))
    deps.update(target.dependencies)
    return deps

  def _thrift_dependencies_for_target(self, target):
    dep_info = self._resolved_dep_info
    target_declares_service = any(self._declares_service(source)
                                  for source in target.sources_relative_to_buildroot())
    language = self._thrift_defaults.language(target)

    if target_declares_service:
      return dep_info.service[language]
    else:
      return dep_info.structs[language]

  @memoized_property
  def _resolved_dep_info(self):
    return ScroogeGen.DepInfo(self._resolve_deps(self.get_options().service_deps),
                              self._resolve_deps(self.get_options().structs_deps))

  @property
  def _copy_target_attributes(self):
    return ['provides', 'strict_deps']




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: ContribPlugin(BuildFileAddress(contrib/scrooge/src/python/pants/contrib/scrooge/BUILD, plugin))

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: MacOS :: MacOS X',
                       'Operating System :: POSIX :: Linux',
                       'Programming Language :: Python',
                       'Topic :: Software Development :: Build Tools',
                       'Topic :: Software Development :: Code Generators'],
    'description': 'Scrooge thrift generator pants plugins.',
    'entry_points': {   'pantsbuild.plugin': [   'register_goals = pants.contrib.scrooge.register:register_goals']},
    'install_requires': [   'twitter.common.collections<0.4,>=0.3.1',
                            'pantsbuild.pants==1.3.0'],
    'license': 'Apache License, Version 2.0',
    'long_description': 'Pants is an Apache2 licensed build tool written in Python.\n\nThe latest documentation can be found `here <http://pantsbuild.org/>`_.\n\n1.3.x Stable Releases\n=====================\n\nThis document describes releases leading up to the ``1.3.x`` ``stable`` series.\n\n1.3.0 (06/06/2017)\n------------------\n\nThe ``1.3.0`` stable release, with no addition changes since the rc4 release!\n\nA quick summary of exciting changes since the ``1.2.x`` branch:\n\n* Pants\' python backend has been overhauled and modernized to support caching, and to lower the\n  latency of re-running tests after a code edit.\n* Pants now uses a `native code \'engine\' <https://docs.google.com/document/d/1C64MreDeVoZAl3HrqtWUVE-qnj3MyWW0NQ52xejjaWw/edit?usp=sharing>`_\n  implemented in rust, which unlocks significant performance improvements in the future.\n* A daemon that caches BUILD file parsing and file fingerprinting (and more to follow!) is now `ready\n  for alpha testing <https://github.com/pantsbuild/pants/blob/b5eaf05ef533df4c5f93a3728107a27f25726941/pants.daemon.ini>`_\n  (with `two significant caveats <http://www.pantsbuild.org/invoking.html#the-pants-daemon>`_).\n* A default set of `implicit_sources <https://rbcommons.com/s/twitter/r/4300>`_ can now be\n  manually enabled to further reduce BUILD file boilerplate. This option should become the\n  default in the next stable release.\n* The Zinc incremental scala compiler is now at 1.0.0, with improved incremental behaviour.\n* Style-related tasks that used to be in the `compile` goal have moved to a new `lint` goal.\n* The `fmt` goal was added, with support for scalafmt, `gofmt`, and python `isort` to start.\n* `Support for yarnpkg <https://github.com/pantsbuild/pants/pull/4255>`_ in the node contrib module.\n* Support for `relative \'jar\' declarations <https://github.com/pantsbuild/pants/pull/4279>`_, to\n  allow for committing jars in your repo (if that\'s your bag!)\n* A contrib module for `errorprone <https://github.com/pantsbuild/pants/pull/4163>`_ was added.\n* Direct support for `scalatest` tests in the pants JVM test runner.\n\nSee the rest of this page for the complete list of changes.\n\n1.3.0rc4 (05/27/2017)\n---------------------\n\nThe fifth release candidate for 1.3.0, with fixes for two more user reported issues.\n\nBugfixes\n~~~~~~~~\n\n* Temporarily restore recursive behaviour for bundle filesets (#4630)\n  `PR #4630 <https://github.com/pantsbuild/pants/pull/4630>`_\n  `PR #4624 <https://github.com/pantsbuild/pants/pull/4624>`_\n  `PR #4624 <https://github.com/pantsbuild/pants/pull/4624>`_\n\n* Check that test case attribute exists in junit xml file before converting it (#4623)\n  `Issue #4619 <https://github.com/pantsbuild/pants/issues/4619>`_\n  `PR #4623 <https://github.com/pantsbuild/pants/pull/4623>`_\n\n\n1.3.0rc3 (05/23/2017)\n---------------------\n\nThe fourth release candidate for 1.3.0, which fixes an issue that caused spurious cache misses\ncross platform in the v2 engine.\n\nBugfixes\n~~~~~~~~\n\n* [engine] Check for duplicate deps in v2 graph construction. (#4616)\n  `PR #4616 <https://github.com/pantsbuild/pants/pull/4616>`_\n\n* Improve Snapshot determinism (#4614)\n  `PR #4614 <https://github.com/pantsbuild/pants/pull/4614>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Remove Oracle Java6, which is now 404ing in Travis. (#4615)\n  `PR #4615 <https://github.com/pantsbuild/pants/pull/4615>`_\n\n1.3.0rc2 (05/19/2017)\n---------------------\n\nThe third release candidate for 1.3.0. This fixes a few more issues discovered during very\nthorough testing by the community. Thank you everyone!\n\nAPI Changes\n~~~~~~~~~~~\n\n* Change method of reporting target data (#4593)\n  `PR #4593 <https://github.com/pantsbuild/pants/pull/4593>`_\n\n* Include API that will store target info in run_tracker (#4561)\n  `PR #4561 <https://github.com/pantsbuild/pants/pull/4561>`_\n\n* Switch to a conditional deprecation for the list-targets behaviour change. (#4600)\n  `PR #4600 <https://github.com/pantsbuild/pants/pull/4600>`_\n\nBugfixes\n~~~~~~~~\n\n* Revert "Enable --compile-zinc-use-classpath-jars by default" (#4607)\n  `PR #4607 <https://github.com/pantsbuild/pants/pull/4607>`_\n\n* Pass env vars through in ./pants run for python (#4606)\n  `PR #4606 <https://github.com/pantsbuild/pants/pull/4606>`_\n\n* Fix broken export-classpath (#4603)\n  `PR #4603 <https://github.com/pantsbuild/pants/pull/4603>`_\n\n* Fix export-classpaths exclude behavior (#4592)\n  `PR #4592 <https://github.com/pantsbuild/pants/pull/4592>`_\n\n* Fix splitting of the build_flags for golang. (#4580)\n  `PR #4580 <https://github.com/pantsbuild/pants/pull/4580>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Don\'t register newpython tasks in the oldpython backend (#4602)\n  `PR #4602 <https://github.com/pantsbuild/pants/pull/4602>`_\n\n1.3.0rc1 (05/12/2017)\n---------------------\n\nThe second release candidate for 1.3.0! Thanks to excellent reports from a bunch of folks,\na few issues have been shaken out of the new codepaths introduced by the v2 engine.\n\nIt\'s possible that this will be the final release candidate for 1.3.0: please help give it\na thorough testing to help discover any more issues!\n\nAPI Changes\n~~~~~~~~~~~\n\n* Support "exports" for thrift targets (#4564)\n  `PR #4564 <https://github.com/pantsbuild/pants/pull/4564>`_\n\n* Make setup_py tasks provide \'python_dists\' product. (#4498)\n  `PR #4498 <https://github.com/pantsbuild/pants/pull/4498>`_\n\n* Include transitive Resources targets in PrepareResources. (#4569)\n  `PR #4569 <https://github.com/pantsbuild/pants/pull/4569>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix built-in macros for the mutable ParseContext (#4583)\n  `PR #4583 <https://github.com/pantsbuild/pants/pull/4583>`_\n\n* Exclude only roots for exclude-target-regexp in v2 (#4578)\n  `PR #4578 <https://github.com/pantsbuild/pants/pull/4578>`_\n\n* Fix a pytest path mangling bug. (#4565)\n  `PR #4565 <https://github.com/pantsbuild/pants/pull/4565>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Specify a workunit for node.js test and run. (#4572)\n  `PR #4572 <https://github.com/pantsbuild/pants/pull/4572>`_\n\n* [engine] Don\'t recreate a graph just for validation (#4566)\n  `PR #4566 <https://github.com/pantsbuild/pants/pull/4566>`_\n\n\n1.3.0rc0 (05/08/2017)\n---------------------\n\nThe first release candidate for the 1.3.0 stable release branch! Almost 7 months\nin the making, this release brings a huge set of changes, which will be summarized\nfor the 1.3.0 final release.\n\nPlease test this release candidate to help ensure a stable stable 1.3.0 release!\n\nAPI Changes\n~~~~~~~~~~~\n\n* [engine] Deprecate and replace `traversable_dependency_specs`. (#4542)\n  `PR #4542 <https://github.com/pantsbuild/pants/pull/4542>`_\n\n* Move scalastyle and java checkstyle into the `lint` goal (#4540)\n  `PR #4540 <https://github.com/pantsbuild/pants/pull/4540>`_\n\nBugfixes\n~~~~~~~~\n\n* Warn when implicit_sources would be used, but is disabled (#4559)\n  `PR #4559 <https://github.com/pantsbuild/pants/pull/4559>`_\n\n* Ignore dot-directories by default (#4556)\n  `PR #4556 <https://github.com/pantsbuild/pants/pull/4556>`_\n\n* Dockerize native engine builds. (#4554)\n  `PR #4554 <https://github.com/pantsbuild/pants/pull/4554>`_\n\n* Make "changed" tasks work with deleted files (#4546)\n  `PR #4546 <https://github.com/pantsbuild/pants/pull/4546>`_\n\n* Fix tag builds after the more-complete isort edit. (#4532)\n  `PR #4532 <https://github.com/pantsbuild/pants/pull/4532>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [engine] Support tracebacks in engine traces; only show them w/ flag (#4549)\n  `PR #4549 <https://github.com/pantsbuild/pants/pull/4549>`_\n\n* Fix two usages of Address.build_file that avoided detection during the deprecation. (#4538)\n  `PR #4538 <https://github.com/pantsbuild/pants/pull/4538>`_\n\nDocumentation Updates\n~~~~~~~~~~~~~~~~~~~~~\n\n* Update target scope docs (#4553)\n  `PR #4553 <https://github.com/pantsbuild/pants/pull/4553>`_\n\n* [engine] use rust doc comments instead of javadoc style comments (#4550)\n  `PR #4550 <https://github.com/pantsbuild/pants/pull/4550>`_\n\n1.3.0.dev19 (4/28/2017)\n-----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Add support for \'deployable_archives\' for go and cpp rules. (#4518)\n  `PR #4518 <https://github.com/pantsbuild/pants/pull/4518>`_\n\n* Deprecate `BuildFileAddress.build_file` (#4511)\n  `PR #4511 <https://github.com/pantsbuild/pants/pull/4511>`_\n\n* Make usage of pantsd imply usage of watchman. (#4512)\n  `PR #4512 <https://github.com/pantsbuild/pants/pull/4512>`_\n\n* Enable --compile-zinc-use-classpath-jars by default (#4525)\n  `PR #4525 <https://github.com/pantsbuild/pants/pull/4525>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix the kythe bootclasspath. (#4527)\n  `PR #4527 <https://github.com/pantsbuild/pants/pull/4527>`_\n\n* Revert the zinc `1.0.0-X7` upgrade (#4510)\n  `PR #4510 <https://github.com/pantsbuild/pants/pull/4510>`_\n\n* Invoke setup-py using an interpreter that matches the target. (#4482)\n  `PR #4482 <https://github.com/pantsbuild/pants/pull/4482>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [pantsd] Ensure rust panics surface in output or daemon logs (#4522)\n  `PR #4522 <https://github.com/pantsbuild/pants/pull/4522>`_\n\n* Make the release script more idempotent. (#4504)\n  `PR #4504 <https://github.com/pantsbuild/pants/pull/4504>`_\n\n* [engine] pass on ResolveErrors during address injection (#4523)\n  `PR #4523 <https://github.com/pantsbuild/pants/pull/4523>`_\n\n* [engine] Improve error messages for missing/empty dirs (#4517)\n  `PR #4517 <https://github.com/pantsbuild/pants/pull/4517>`_\n\n* Render failed junit tests with no target owner. (#4521)\n  `PR #4521 <https://github.com/pantsbuild/pants/pull/4521>`_\n\n* [engine] Better error messages for missing targets (#4509)\n  `PR #4509 <https://github.com/pantsbuild/pants/pull/4509>`_\n\n* Options should only default to --color=True when sys.stdout isatty (#4503)\n  `PR #4503 <https://github.com/pantsbuild/pants/pull/4503>`_\n\nDocumentation Updates\n~~~~~~~~~~~~~~~~~~~~~\n\n* Add a scala specs2 example (#4516)\n  `PR #4516 <https://github.com/pantsbuild/pants/pull/4516>`_\n\n\n1.3.0.dev18 (4/21/2017)\n-----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Create a lint goal and put checkstyle tasks in it. (#4481)\n  `PR #4481 <https://github.com/pantsbuild/pants/pull/4481>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix some incorrectly formatted dev release semvers. (#4501)\n  `PR #4501 <https://github.com/pantsbuild/pants/pull/4501>`_\n\n* Make go targets work with v2 changed. (#4500)\n  `PR #4500 <https://github.com/pantsbuild/pants/pull/4500>`_\n\n* Fix pytest fixture registration bug. (#4497)\n  `PR #4497 <https://github.com/pantsbuild/pants/pull/4497>`_\n\n* Don\'t trigger deprecated scope warnings for options from the DEFAULT section (#4487)\n  `PR #4487 <https://github.com/pantsbuild/pants/pull/4487>`_\n\n* Ensure that incomplete scalac plugin state doesn\'t get memoized. (#4480)\n  `PR #4480 <https://github.com/pantsbuild/pants/pull/4480>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [engine] Skip re-creating copy of address if no variants (#4032)\n  `PR #4032 <https://github.com/pantsbuild/pants/pull/4032>`_\n\n* Default `Fetcher.ProgressListener` to stderr. (#4499)\n  `PR #4499 <https://github.com/pantsbuild/pants/pull/4499>`_\n\n* A contrib plugin to run the Kythe indexer on Java source. (#4457)\n  `PR #4457 <https://github.com/pantsbuild/pants/pull/4457>`_\n\n* Keep failed target mapping free from `None` key. (#4493)\n  `PR #4493 <https://github.com/pantsbuild/pants/pull/4493>`_\n\n* Bring back --no-fast mode in pytest run. (#4491)\n  `PR #4491 <https://github.com/pantsbuild/pants/pull/4491>`_\n\n* [engine] Use enum for RuleEdges keys, add factory for Selects w/o variants (#4461)\n  `PR #4461 <https://github.com/pantsbuild/pants/pull/4461>`_\n\n* Bump scala platform versions to 2.11.11 and 2.12.2 (#4488)\n  `PR #4488 <https://github.com/pantsbuild/pants/pull/4488>`_\n\n* Get rid of the \'2\' registrations of the new python tasks. (#4486)\n  `PR #4486 <https://github.com/pantsbuild/pants/pull/4486>`_\n\n* Make pytest report sources paths relative to the buildroot. (#4472)\n  `PR #4472 <https://github.com/pantsbuild/pants/pull/4472>`_\n\nDocumentation Updates\n~~~~~~~~~~~~~~~~~~~~~\n\n* [docs] fix broken link to certifi (#3508)\n  `PR #3508 <https://github.com/pantsbuild/pants/pull/3508>`_\n\n* [docs] Fix links in Go README (#3719)\n  `PR #3719 <https://github.com/pantsbuild/pants/pull/3719>`_\n\n* Update globs.md (#4476)\n  `PR #4476 <https://github.com/pantsbuild/pants/pull/4476>`_\n\n* Fix some compiler plugin documentation nits. (#4462)\n  `PR #4462 <https://github.com/pantsbuild/pants/pull/4462>`_\n\n* Convert readthedocs link for their .org -> .io migration for hosted projects (#3542)\n  `PR #3542 <https://github.com/pantsbuild/pants/pull/3542>`_\n\n\n1.3.0.dev17 (4/15/2017)\n-----------------------\nA weekly unstable release, highlighted by setting the new python backend as the default.\n\nAPI Changes\n~~~~~~~~~~~\n* Upgrade pants to current versions of pytest et al. (#4410)\n  `PR #4410 <https://github.com/pantsbuild/pants/pull/4410>`_\n\n* Add ParseContext singleton helper (#4466)\n  `PR #4466 <https://github.com/pantsbuild/pants/pull/4466>`_\n\n* Make the new python backend the default. (#4441)\n  `PR #4441 <https://github.com/pantsbuild/pants/pull/4441>`_\n\nBugfixes\n~~~~~~~~\n* Correctly inject Yarn into the Node path when it is in use (#4455)\n  `PR #4455 <https://github.com/pantsbuild/pants/pull/4455>`_\n\n* Fix resource loading issue in the python eval task. (#4452)\n  `PR #4452 <https://github.com/pantsbuild/pants/pull/4452>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n* [engine] Use RuleGraph for task lookup instead of Tasks (#4371)\n  `PR #4371 <https://github.com/pantsbuild/pants/pull/4371>`_\n\n* Re-use pre-built Linux engine binaries for bintray upload. (#4454)\n  `PR #4454 <https://github.com/pantsbuild/pants/pull/4454>`_\n\n* Replace `indices` with `indexes` in docs (#4453)\n  `PR #4453 <https://github.com/pantsbuild/pants/pull/4453>`_\n\n* Avoid re-walking for every target root in minimize (#4463)\n  `PR #4463 <https://github.com/pantsbuild/pants/pull/4463>`_\n\n\n1.3.0.dev16 (4/08/2017)\n-----------------------\nA weekly unstable release.\n\nThis release brings the new `pantsbuild.pants.contrib.jax_ws\n<https://github.com/pantsbuild/pants/tree/master/contrib/jax_ws>`_ plugin that can generate Java\nclient stubs from WSDL sources. Thanks to Chris Heisterkamp for this!\n\nThe release also pulls in a few fixes for python requirement resolution in the PEX library used by\npants. In the past, the python-setup.resolver_allow_prereleases configuration option would not\nalways be resepected; it now is. Additionally, a longstanding bug in transitive requirement\nresolution that would lead to erroneous \'Ambiguous resolvable\' errors has now been fixed. Thanks to\nTodd Gardner and Nathan Butler for these fixes!\n\nNew Features\n~~~~~~~~~~~~\n\n* Add JAX-WS plugin to generate client stub files from WSDL files (#4411)\n  `PR #4411 <https://github.com/pantsbuild/pants/pull/4411>`_\n\nAPI Changes\n~~~~~~~~~~~\n\n* Disable unused deps by default (#4440)\n  `PR #4440 <https://github.com/pantsbuild/pants/pull/4440>`_\n\n* Bump pex version to 1.2.6 (#4442)\n  `PR #4442 <https://github.com/pantsbuild/pants/pull/4442>`_\n\n* Upgrade to pex 1.2.5. (#4434)\n  `PR #4434 <https://github.com/pantsbuild/pants/pull/4434>`_\n\n* Update 3rdparty jars: args4j to 2.33, jsr305 to 3.0.2, easymock to 3.4, burst-junit4 to 1.1.1, commons-io to 2.5, and mockito-core to 2.7.21 (#4421)\n  `PR #4421 <https://github.com/pantsbuild/pants/pull/4421>`_\n\nBugfixes\n~~~~~~~~\n\n* Default --resolver-allow-prereleases to None. (#4445)\n  `PR #4445 <https://github.com/pantsbuild/pants/pull/4445>`_\n\n* Fully hydrate a BuildGraph for the purposes of ChangedCalculator. (#4424)\n  `PR #4424 <https://github.com/pantsbuild/pants/pull/4424>`_\n\n* Upgrade zinc to `1.0.0-X7` (python portion) (#4419)\n  `Issue #75 <https://github.com/sbt/util/issues/75>`_\n  `Issue #218 <https://github.com/sbt/zinc/issues/218>`_\n  `PR #4419 <https://github.com/pantsbuild/pants/pull/4419>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [engine] New shared impl (#4429)\n  `PR #4429 <https://github.com/pantsbuild/pants/pull/4429>`_\n  `PR #442) <https://github.com/alexcrichton/futures-rs/pull/442)>`_\n\n* Speed up changed task when backed by v2 engine. (#4422)\n  `PR #4422 <https://github.com/pantsbuild/pants/pull/4422>`_\n\n1.3.0.dev15 (4/03/2017)\n-----------------------\nA weekly unstable release, delayed by a week!\n\nThis release contains multiple significant new features, including the "exports" literal\non JVM targets (to better support common cases in repositories using "strict_deps"), the\ninitial release of the new python backend with caching support, a new "outdated.ivy" goal\nto report which JVM dependencies are out of date, speedups for go builds, and last but not\nleast: the first release with the v2 engine enabled by default (to enable stabilization of\nthe pants daemon before the 1.3.x stable releases).\n\nThanks to the contributors!\n\nNew Features\n~~~~~~~~~~~~\n\n* Add outdated.ivy command that looks for 3rd party jar updates with Ivy (#4386)\n  `PR #4386 <https://github.com/pantsbuild/pants/pull/4386>`_\n\n* Implement exports literal in jvm_target (#4329)\n  `PR #4329 <https://github.com/pantsbuild/pants/pull/4329>`_\n\n* Make jar_library target export all its dependencies (#4395)\n  `PR #4395 <https://github.com/pantsbuild/pants/pull/4395>`_\n\n* A temporary `python2` backend with just the new python pipeline tasks. (#4378)\n  `PR #4378 <https://github.com/pantsbuild/pants/pull/4378>`_\n\n* [engine] include rule graph in dot files generated with --visualize-to (#4367)\n  `PR #4367 <https://github.com/pantsbuild/pants/pull/4367>`_\n\n* Speed up typical go builds. (#4362)\n  `PR #4362 <https://github.com/pantsbuild/pants/pull/4362>`_\n\n* Enable v2 engine by default. (#4340)\n  `PR #4340 <https://github.com/pantsbuild/pants/pull/4340>`_\n\nAPI Changes\n~~~~~~~~~~~\n\n* Use released ivy-dependency-update-checker jar tool for outdated.ivy command (#4406)\n  `PR #4406 <https://github.com/pantsbuild/pants/pull/4406>`_\n\n* Improve our use of gofmt. (#4379)\n  `PR #4379 <https://github.com/pantsbuild/pants/pull/4379>`_\n\n* Bump the default scala 2.12 minor version to 2.12.1. (#4383)\n  `PR #4383 <https://github.com/pantsbuild/pants/pull/4383>`_\n\nBugfixes\n~~~~~~~~\n\n* [pantsd] Lazily initialize `CpuPool` for `Core` and `PosixFS` to address `SchedulerService` crash on Linux. (#4412)\n  `PR #4412 <https://github.com/pantsbuild/pants/pull/4412>`_\n\n* [pantsd] Address pantsd-runner hang on Linux and re-enable integration test. (#4407)\n  `PR #4407 <https://github.com/pantsbuild/pants/pull/4407>`_\n\n* Switch the new PytestRun task to use junitxml output. (#4403)\n  `Issue #3837 <https://github.com/pantsbuild/pants/issues/3837>`_\n  `PR #4403 <https://github.com/pantsbuild/pants/pull/4403>`_\n\n* [contrib/go] only pass go sources to gofmt (#4402)\n  `PR #4402 <https://github.com/pantsbuild/pants/pull/4402>`_\n\n* Remove Address/BuildFileAddress ambiguity and fix list-owners (#4399)\n  `PR #4399 <https://github.com/pantsbuild/pants/pull/4399>`_\n\n* Avoid creating deprecated resources in JavaAgent\'s constructor (#4400)\n  `PR #4400 <https://github.com/pantsbuild/pants/pull/4400>`_\n\n* Invalidate all go compiles when the go version changes. (#4382)\n  `PR #4382 <https://github.com/pantsbuild/pants/pull/4382>`_\n\n* Repair handling on resources kwargs for changed. (#4396)\n  `PR #4396 <https://github.com/pantsbuild/pants/pull/4396>`_\n\n* python-binary-create task maps all product directories to the same target (#4390)\n  `PR #4390 <https://github.com/pantsbuild/pants/pull/4390>`_\n\n* Fix Go source excludes; Cleanup old filespec matching (#4350)\n  `PR #4350 <https://github.com/pantsbuild/pants/pull/4350>`_\n\n* inserted a www. into some pantsbuild links to un-break them (#4388)\n  `PR #4388 <https://github.com/pantsbuild/pants/pull/4388>`_\n\n* Switch to using the new PythonEval task instead of the old one. (#4374)\n  `PR #4374 <https://github.com/pantsbuild/pants/pull/4374>`_\n\n* Adding pragma back in the default coverage config (#4232)\n  `PR #4232 <https://github.com/pantsbuild/pants/pull/4232>`_\n\n* decode compile logs (#4368)\n  `PR #4368 <https://github.com/pantsbuild/pants/pull/4368>`_\n\n* Skip cycle detection test (#4361)\n  `PR #4361 <https://github.com/pantsbuild/pants/pull/4361>`_\n\n* [engine] Fix whitelisting of files in `pants_ignore` (#4357)\n  `PR #4357 <https://github.com/pantsbuild/pants/pull/4357>`_\n\n* Revert the shared workaround (#4354)\n  `PR #4348 <https://github.com/pantsbuild/pants/pull/4348>`_\n  `PR #4354 <https://github.com/pantsbuild/pants/pull/4354>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Cleanup and give better debug output for exclude patterns in findbugs and errorprone (#4408)\n  `PR #4408 <https://github.com/pantsbuild/pants/pull/4408>`_\n\n* [engine] Rules as decorators (#4369)\n  `PR #4369 <https://github.com/pantsbuild/pants/pull/4369>`_\n\n* [engine] Move snapshots from /tmp to pants workdir. (#4373)\n  `PR #4373 <https://github.com/pantsbuild/pants/pull/4373>`_\n\n* [engine] Init Tasks before Scheduler (#4381)\n  `PR #4381 <https://github.com/pantsbuild/pants/pull/4381>`_\n\n* TravisCI tuning. (#4385)\n  `PR #4385 <https://github.com/pantsbuild/pants/pull/4385>`_\n\n* Switch the pants repo entirely over to the new python pipeline. (#4316)\n  `PR #4316 <https://github.com/pantsbuild/pants/pull/4316>`_\n\n* Fix missing deps. (#4372)\n  `PR #4372 <https://github.com/pantsbuild/pants/pull/4372>`_\n\n* A PythonEval task that uses the new pipeline. (#4341)\n  `PR #4341 <https://github.com/pantsbuild/pants/pull/4341>`_\n\n* Create a pants.init package. (#4356)\n  `PR #4356 <https://github.com/pantsbuild/pants/pull/4356>`_\n\n* [engine] short circuit native engine build failures (#4353)\n  `PR #4353 <https://github.com/pantsbuild/pants/pull/4353>`_\n\n* Check for stale native_engine_version. (#4360)\n  `PR #4360 <https://github.com/pantsbuild/pants/pull/4360>`_\n\n* [engine] Improving performance by iteratively expanding products within SelectTransitive (#4349)\n  `PR #4349 <https://github.com/pantsbuild/pants/pull/4349>`_\n\n* Move all logic out of Context (#4343)\n  `PR #4343 <https://github.com/pantsbuild/pants/pull/4343>`_\n\n* Add support for subprojects in v2 (#4346)\n  `PR #4346 <https://github.com/pantsbuild/pants/pull/4346>`_\n\n* Fix missing and circular deps. (#4345)\n  `Issue #4138 <https://github.com/pantsbuild/pants/issues/4138>`_\n  `PR #4345 <https://github.com/pantsbuild/pants/pull/4345>`_\n\n1.3.0.dev14 (3/17/2017)\n-----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* [pantsd] Add an option to configure the watchman startup timeout. (#4332)\n  `PR #4332 <https://github.com/pantsbuild/pants/pull/4332>`_\n\n* Relativize jar_dependency.base_path (#4326)\n  `PR #4326 <https://github.com/pantsbuild/pants/pull/4326>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix bad import from race commits (#4335)\n  `PR #4335 <https://github.com/pantsbuild/pants/pull/4335>`_\n\n* Misc fixes to python tasks: (#4323)\n  `PR #4323 <https://github.com/pantsbuild/pants/pull/4323>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Fix product mapping for ivy resolve when libraries are not jar files (#4339)\n  `PR #4339 <https://github.com/pantsbuild/pants/pull/4339>`_\n\n* Refactor the new SelectInterpreter and GatherSources tasks. (#4337)\n  `PR #4337 <https://github.com/pantsbuild/pants/pull/4337>`_\n\n* Lock down the native-engine-version (#4338)\n  `PR #4338 <https://github.com/pantsbuild/pants/pull/4338>`_\n\n* [engine] Inline execution of Select/Dependencies/Projection/Literal (#4331)\n  `PR #4331 <https://github.com/pantsbuild/pants/pull/4331>`_\n\n* Upgrade to mock 2.0.0. (#4336)\n  `PR #4336 <https://github.com/pantsbuild/pants/pull/4336>`_\n\n* [engine] Improve memory layout for Graph (#4333)\n  `PR #4333 <https://github.com/pantsbuild/pants/pull/4333>`_\n\n* [engine] Split SelectDependencies into SelectDependencies and SelectTransitive (#4334)\n  `PR #4334 <https://github.com/pantsbuild/pants/pull/4334>`_\n\n* Simplify PythonSetup usage (#4328)\n  `PR #4328 <https://github.com/pantsbuild/pants/pull/4328>`_\n\n* Bump native engine version. (#4330)\n  `PR #4330 <https://github.com/pantsbuild/pants/pull/4330>`_\n\n* [engine] Move to new-style CFFI callbacks. (#4324)\n  `PR #4324 <https://github.com/pantsbuild/pants/pull/4324>`_\n\n* Profile the pants invocations in integration tests. (#4325)\n  `PR #4325 <https://github.com/pantsbuild/pants/pull/4325>`_\n\n1.3.0.dev13 (3/10/2017)\n-----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Bump pex version to latest. (#4314)\n  `PR #4314 <https://github.com/pantsbuild/pants/pull/4314>`_\n\nNew Features\n~~~~~~~~~~~~\n\n* Binary builder task for the new python pipeline. (#4313)\n  `PR #4313 <https://github.com/pantsbuild/pants/pull/4313>`_\n\n* [engine] rm python graphmaker; create dot formatted display (#4295)\n  `PR #4295 <https://github.com/pantsbuild/pants/pull/4295>`_\n\n* A setup_py task for the new python pipeline. (#4308)\n  `PR #4308 <https://github.com/pantsbuild/pants/pull/4308>`_\n\nBugfixes\n~~~~~~~~\n\n* scrooge_gen task copy strict_deps field (#4321)\n  `PR #4321 <https://github.com/pantsbuild/pants/pull/4321>`_\n\n* [jvm-compile] Copy compile classpath into runtime classpath even if already defined (#4310)\n  `PR #4310 <https://github.com/pantsbuild/pants/pull/4310>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Fix reliance on symlinks in testdata. (#4320)\n  `PR #4320 <https://github.com/pantsbuild/pants/pull/4320>`_\n\n* Introduce SUPPRESS_LABEL on workunit\'s console output and have thrift-linter and jar-tool adopt it (#4318)\n  `PR #4318 <https://github.com/pantsbuild/pants/pull/4318>`_\n\n1.3.0.dev12 (3/3/2017)\n----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Completely revamp how we support JVM compiler plugins. (#4287)\n  `PR #4287 <https://github.com/pantsbuild/pants/pull/4287>`_\n\n\nNew Features\n~~~~~~~~~~~~\n\n* A PytestRun task for the new Python pipeline. (#4252)\n  `PR #4252 <https://github.com/pantsbuild/pants/pull/4252>`_\n\n* Add ability to specify subprojects (#4088)\n  `PR #4088 <https://github.com/pantsbuild/pants/pull/4088>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix missed native_engine_version.\n  `Commit cbdb97515 <https://github.com/pantsbuild/pants/commit/cbdb97515>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [engine] Rust IO (#4265)\n  `PR #4265 <https://github.com/pantsbuild/pants/pull/4265>`_\n\n* [engine] Support implicit sources in v2 engine (#4294)\n  `PR #4294 <https://github.com/pantsbuild/pants/pull/4294>`_\n\n* SelectLiteral isn\'t tied to the requester\'s subject: it has its own. (#4293)\n  `PR #4293 <https://github.com/pantsbuild/pants/pull/4293>`_\n\n* Include Javascript files in JVM binary (#4264)\n  `PR #4264 <https://github.com/pantsbuild/pants/pull/4264>`_\n\n* Update errorprone to version 2.0.17 (#4291)\n  `PR #4291 <https://github.com/pantsbuild/pants/pull/4291>`_\n\n* node_modules and node_test support yarnpkg as package manager (#4255)\n  `PR #4255 <https://github.com/pantsbuild/pants/pull/4255>`_\n\n\n1.3.0.dev11 (2/24/2017)\n-----------------------\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Support local jar with relative path in JarDependency (#4279)\n  `PR #4279 <https://github.com/pantsbuild/pants/pull/4279>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Upgrade default jarjar to 1.6.4. (#4271)\n  `Issue #26 <https://github.com/pantsbuild/jarjar/issues/26>`_\n  `PR #4271 <https://github.com/pantsbuild/pants/pull/4271>`_\n\n* Memoize validation of deprecated versions (#4273)\n  `PR #4273 <https://github.com/pantsbuild/pants/pull/4273>`_\n\n* [engine] Remove type_id field from Value (#4274)\n  `PR #4274 <https://github.com/pantsbuild/pants/pull/4274>`_\n\n* [New Python Pipeline] Add resources to PEXes correctly. (#4275)\n  `PR #4275 <https://github.com/pantsbuild/pants/pull/4275>`_\n\n* Upgrade default go to 1.8. (#4272)\n  `PR #4272 <https://github.com/pantsbuild/pants/pull/4272>`_\n\n* Fix missed native_engine_version commit.\n  `Commit d966f9592 <https://github.com/pantsbuild/pants/commit/d966f9592fba2040429fc8a64f8aa4deb5e61f2c>`_\n\n* Make options fingerprinting very difficult to disable (#4262)\n  `PR #4262 <https://github.com/pantsbuild/pants/pull/4262>`_\n\n* Bump pex requirement to 1.2.3 (#4277)\n  `PR #4277 <https://github.com/pantsbuild/pants/pull/4277>`_\n\n* Strip the root-level __init__.py that apache thrift generates. (#4281)\n  `PR #4281 <https://github.com/pantsbuild/pants/pull/4281>`_\n\n* Small tweak to the Dockerfile. (#4263)\n  `PR #4263 <https://github.com/pantsbuild/pants/pull/4263>`_\n\n* Make "./pants changed" output correct results when BUILD files are modified (#4282)\n  `PR #4282 <https://github.com/pantsbuild/pants/pull/4282>`_\n\n* [engine] minor clean up `engine.close` usage in `visualizer` (#4284)\n  `PR #4284 <https://github.com/pantsbuild/pants/pull/4284>`_\n\n\n1.3.0.dev10 (2/17/2017)\n-----------------------\n\nBugfixes\n~~~~~~~~\n\n* Treat PythonTarget dependencies on Resources targets appropriately. (#4249)\n  `PR #4249 <https://github.com/pantsbuild/pants/pull/4249>`_\n\n* [engine] fix address node creation in v2 build graph; fix filedeps (#4235)\n  `PR #4235 <https://github.com/pantsbuild/pants/pull/4235>`_\n\n* Repair `Broken pipe` on pantsd thin client execution when piped to a non-draining reader. (#4230)\n  `PR #4230 <https://github.com/pantsbuild/pants/pull/4230>`_\n\nAPI Changes\n~~~~~~~~~~~\n\n* Deprecate Python target resources= and resource_targets=. (#4251)\n  `PR #4251 <https://github.com/pantsbuild/pants/pull/4251>`_\n\n* Deprecate use of resources= in JVM targets. (#4248)\n  `PR #4248 <https://github.com/pantsbuild/pants/pull/4248>`_\n\nNew Features\n~~~~~~~~~~~~\n\n* New python repl task. (#4219)\n  `PR #4219 <https://github.com/pantsbuild/pants/pull/4219>`_\n\n* Add a node bundle goal (#4212)\n  `PR #4212 <https://github.com/pantsbuild/pants/pull/4212>`_\n\n* A task to generate Python code from ANTLR3 grammars.\n  `PR #4206 <https://github.com/pantsbuild/pants/pull/4206>`_\n\nDocumentation\n~~~~~~~~~~~~~\n\n* Fixing grammatical error in why use pants doc page (#4239)\n  `PR #4239 <https://github.com/pantsbuild/pants/pull/4239>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Work around native engine/tag chicken-egg. (#4270)\n  `PR #4270 <https://github.com/pantsbuild/pants/pull/4270>`_\n\n* [engine] Make Graph.get generic and make Externs static (#4261)\n  `PR #4261 <https://github.com/pantsbuild/pants/pull/4261>`_\n\n* A Dockerfile for building a pants development image. (#4260)\n  `PR #4260 <https://github.com/pantsbuild/pants/pull/4260>`_\n\n* [engine] Use Value in invoke_runnable as the function instead of Function (#4258)\n  `PR #4258 <https://github.com/pantsbuild/pants/pull/4258>`_\n\n* [engine] `Storage` clean ups  (#4257)\n  `PR #4257 <https://github.com/pantsbuild/pants/pull/4257>`_\n\n* [engine] remove Field type in favor of using String directly (#4256)\n  `PR #4256 <https://github.com/pantsbuild/pants/pull/4256>`_\n\n* Remove our use of resources= and resource_targets= in python targets. (#4250)\n  `PR #4250 <https://github.com/pantsbuild/pants/pull/4250>`_\n\n* Get rid of resources=[] stanzas in our JVMTargets. (#4247)\n  `PR #4247 <https://github.com/pantsbuild/pants/pull/4247>`_\n\n* Change engine visual graph layout from LR to TB (#4245)\n  `PR #4245 <https://github.com/pantsbuild/pants/pull/4245>`_\n\n* Simplify ci script test running stanzas. (#4209)\n  `PR #4209 <https://github.com/pantsbuild/pants/pull/4209>`_\n\n* [engine] Porting validation to Rust pt ii (#4243)\n  `PR #4243 <https://github.com/pantsbuild/pants/pull/4243>`_\n\n* Require dev-suffixed deprecation versions (#4216)\n  `PR #4216 <https://github.com/pantsbuild/pants/pull/4216>`_\n\n* [engine] Begin port of engine rule graph validation to Rust (#4227)\n  `PR #4227 <https://github.com/pantsbuild/pants/pull/4227>`_\n\n* Derive object id used in the native context from object\'s content (#4233)\n  `PR #4233 <https://github.com/pantsbuild/pants/pull/4233>`_\n\n* [engine] Use futures for scheduling (#4221)\n  `PR #4221 <https://github.com/pantsbuild/pants/pull/4221>`_\n\n* Add a \'current\' symlink to the task-versioned prefix of the workdir. (#4220)\n  `PR #4220 <https://github.com/pantsbuild/pants/pull/4220>`_\n\n* Improve BUILD file matching in the v2 path. (#4226)\n  `PR #4226 <https://github.com/pantsbuild/pants/pull/4226>`_\n\n* Batch address injections in dependees task. (#4222)\n  `PR #4222 <https://github.com/pantsbuild/pants/pull/4222>`_\n\n\n1.3.0.dev9 (1/27/2017)\n----------------------\n\nBugfixes\n~~~~~~~~\n\n* Removes the slf4j implementation from the classpath when running Cobertura (#4198)\n  `PR #4198 <https://github.com/pantsbuild/pants/pull/4198>`_\n\n* Make open_zip print realpath when raising BadZipfile. (#4186)\n  `PR #4186 <https://github.com/pantsbuild/pants/pull/4186>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Shard testprojects integration tests (#4205)\n  `PR #4205 <https://github.com/pantsbuild/pants/pull/4205>`_\n\n* Resolve only stable releases by default. (#4201)\n  `PR #4201 <https://github.com/pantsbuild/pants/pull/4201>`_\n\n\n1.3.0.dev8 (1/20/2017)\n----------------------\n\nAPI Changes\n~~~~~~~~~~~\n\n* Bump pex version to 1.1.20 (#4191)\n  `PR #4191 <https://github.com/pantsbuild/pants/pull/4191>`_\n\n* Ban some characters in target name (#4180)\n  `PR #4180 <https://github.com/pantsbuild/pants/pull/4180>`_\n\nNew Features\n~~~~~~~~~~~~\n\n* Scrooge codegen improvements (#4177)\n  `PR #4177 <https://github.com/pantsbuild/pants/pull/4177>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Kill review tooling remnants. (#4192)\n  `PR #4192 <https://github.com/pantsbuild/pants/pull/4192>`_\n\n* Only release native-engine for pants releases. (#4189)\n  `PR #4189 <https://github.com/pantsbuild/pants/pull/4189>`_\n\n* Add some useful tips to the release documentation. (#4183)\n  `PR #4183 <https://github.com/pantsbuild/pants/pull/4183>`_\n\nBugfixes\n~~~~~~~~\n\n* Add __init__.py for tests/python directories (#4193)\n  `PR #4193 <https://github.com/pantsbuild/pants/pull/4193>`_\n\n* Fix `str`-typed options with `int` defaults. (#4184)\n  `PR #4184 <https://github.com/pantsbuild/pants/pull/4184>`_\n\n\n1.3.0.dev7 (1/13/2017)\n----------------------\n\nAPI Changes\n~~~~~~~~~~~\n\n* Upgrade zinc and default scala-platform in pants repo to 2.11 (#4176)\n  `PR #4176 <https://github.com/pantsbuild/pants/pull/4176>`_\n\nNew Features\n~~~~~~~~~~~~\n\n* Add contrib module for Error Prone http://errorprone.info/ (#4163)\n  `PR #4163 <https://github.com/pantsbuild/pants/pull/4163>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* add various codegen packages to default backend packages (#4175)\n  `PR #4175 <https://github.com/pantsbuild/pants/pull/4175>`_\n\n* Suggest missing dependencies from target\'s transitive dependencies (#4171)\n  `PR #4171 <https://github.com/pantsbuild/pants/pull/4171>`_\n\n* Reduce compilation invalidation scope of targets with strict_deps=True (#4143)\n  `PR #4143 <https://github.com/pantsbuild/pants/pull/4143>`_\n\n* Fork to post_stat (#4170)\n  `PR #4170 <https://github.com/pantsbuild/pants/pull/4170>`_\n\nBugfixes\n~~~~~~~~\n\n* fix a small bug in ApacheThriftGenBase class (#4181)\n  `PR #4181 <https://github.com/pantsbuild/pants/pull/4181>`_\n\n\n1.3.0.dev6 (1/06/2017)\n----------------------\n\nAPI Changes\n~~~~~~~~~~~\n\n* Refactor the thrift codegen task. (#4155)\n  `PR #4155 <https://github.com/pantsbuild/pants/pull/4155>`_\n\n* Finish splitting up the codegen backend. (#4147)\n  `PR #4147 <https://github.com/pantsbuild/pants/pull/4147>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Fix import order issues introduced by a previous commit. (#4156)\n  `PR #4156 <https://github.com/pantsbuild/pants/pull/4156>`_\n\n* Bump default nodejs version to 6.9.1 from 6.2.0 (#4161)\n  `PR #4161 <https://github.com/pantsbuild/pants/pull/4161>`_\n\n* Make post_stat async (#4157)\n  `PR #4157 <https://github.com/pantsbuild/pants/pull/4157>`_\n\n* Fix release script owners check.\n  `Commit <https://github.com/pantsbuild/pants/commit/a40234429cc05f6483f91b08f10037429710b5b4>`_\n\n\n1.3.0.dev5 (12/30/2016)\n-----------------------\n\nAPI Changes\n~~~~~~~~~~~\n\n* Upgrade default go to 1.7.4. (#4149)\n  `PR #4149 <https://github.com/pantsbuild/pants/pull/4149>`_\n\nBugfixes\n~~~~~~~~\n\n* Fix instructions for ivy debug logging (#4141)\n  `PR #4141 <https://github.com/pantsbuild/pants/pull/4141>`_\n\n* Handle unicode in classpath entries (#4136)\n  `PR #4136 <https://github.com/pantsbuild/pants/pull/4136>`_\n\n* Ensure that invalid vts have results_dir cleaned before passing to ta\xe2\x80\xa6 (#4139)\n  `PR #4139 <https://github.com/pantsbuild/pants/pull/4139>`_\n\nDocumentation\n~~~~~~~~~~~~~\n\n* [docs] Update the cache section on the Task developer page. (#4152)\n  `PR #4152 <https://github.com/pantsbuild/pants/pull/4152>`_\n\n* Prepare notes for 1.2.1.rc0 (#4146)\n  `PR #4146 <https://github.com/pantsbuild/pants/pull/4146>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Start breaking up the codegen backend. (#4147)\n  `PR #4147 <https://github.com/pantsbuild/pants/pull/4147>`_\n\n* Cleanup unused cffi handles to free memory (#4135)\n  `PR #4135 <https://github.com/pantsbuild/pants/pull/4135>`_\n\n* A new Python run task. (#4142)\n  `PR #4142 <https://github.com/pantsbuild/pants/pull/4142>`_\n\n1.3.0.dev4 (12/08/2016)\n-----------------------\n\nBugfixes\n~~~~~~~~\n\n* Redirect bootstrapping calls in pants binary to stderr (#4131)\n  `PR #4131 <https://github.com/pantsbuild/pants/pull/4131>`_\n\n* Ensure that the protoc root import path is examined first (#4129)\n  `PR #4129 <https://github.com/pantsbuild/pants/pull/4129>`_\n\n* Allow the buildroot to be a source root (#4093)\n  `PR #4093 <https://github.com/pantsbuild/pants/issues/4093>`_\n\n* A flag to add the buildroot to protoc\'s import path (#4122)\n  `PR #4122 <https://github.com/pantsbuild/pants/pull/4122>`_\n\n* Drop libc dependency from native engine (#4124)\n  `PR #4124 <https://github.com/pantsbuild/pants/pull/4124>`_\n\n* Execute traces for all non-Return values (#4118)\n  `PR #4118 <https://github.com/pantsbuild/pants/pull/4118>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* directly invoke runnable from native code (#4128)\n  `PR #4128 <https://github.com/pantsbuild/pants/pull/4128>`_\n\n* Bump pex version.\n\n* [engine] model snapshots in validation, make root rules a dict instead of a set (#4125)\n  `PR #4125 <https://github.com/pantsbuild/pants/pull/4125>`_\n\n* classmap: a jvm console task that outputs mapping from class products to their targets (#4081)\n  `PR #4081 <https://github.com/pantsbuild/pants/pull/4081>`_\n\n* Update bintray deploys to use a shared account. (#4126)\n  `PR #4126 <https://github.com/pantsbuild/pants/pull/4126>`_\n\n* Plumb a configurable worker count to thrift linter. (#4121)\n  `PR #4121 <https://github.com/pantsbuild/pants/pull/4121>`_\n\nDocumentation\n~~~~~~~~~~~~~\n\n* [docs] Add section for building multiplatform python binaries with native dependencies (#4119)\n  `PR #4119 <https://github.com/pantsbuild/pants/pull/4119>`_\n\n\n1.3.0.dev3 (12/02/2016)\n-----------------------\n\nA weekly unstable release.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Bump pex and setuptools to latest. (#4111)\n  `PR #4111 <https://github.com/pantsbuild/pants/pull/4111>`_\n\n* Bump setuptools version. (#4103)\n  `PR #4103 <https://github.com/pantsbuild/pants/pull/4103>`_\n\nBugfixes\n~~~~~~~~\n\n* Update junit-runner to 1.0.17 (#4113)\n  `PR #4113 <https://github.com/pantsbuild/pants/pull/4113>`_\n  `PR #4106 <https://github.com/pantsbuild/pants/pull/4106>`_\n\n* Don\'t exit the JUnitRunner with number of failures because Java will mod the exit code. (#4106)\n  `PR #4106 <https://github.com/pantsbuild/pants/pull/4106>`_\n\n* Allow for using the native engine from-source in another repo (#4105)\n  `PR #4105 <https://github.com/pantsbuild/pants/pull/4105>`_\n\n* Un-publish the `jar` goal. (#4095)\n  `PR #4095 <https://github.com/pantsbuild/pants/pull/4095>`_\n\n* Restore compile-zinc-name-hashing option to follow deprecation cycle (#4091)\n  `PR #4091 <https://github.com/pantsbuild/pants/pull/4091>`_\n\n* Fix a Python requirement resolution test bug. (#4087)\n  `PR #4087 <https://github.com/pantsbuild/pants/pull/4087>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* [engine] Remove variant selecting from native engine (#4108)\n  `PR #4108 <https://github.com/pantsbuild/pants/pull/4108>`_\n\n* Reduce hashing during v2 transitive graph walks (#4109)\n  `PR #4109 <https://github.com/pantsbuild/pants/pull/4109>`_\n\n* Add a native engine release check. (#4096)\n  `PR #4096 <https://github.com/pantsbuild/pants/pull/4096>`_\n\n* Remove coveralls from CI. (#4099)\n  `PR #4099 <https://github.com/pantsbuild/pants/pull/4099>`_\n\n* Run the proto compiler in workunit. (#4092)\n  `PR #4092 <https://github.com/pantsbuild/pants/pull/4092>`_\n\n* Restore propagation of thrown exceptions between rust and python (#4083)\n  `PR #4083 <https://github.com/pantsbuild/pants/pull/4083>`_\n\n* Make `cargo build --release` the default for native engine bootstrapping. (#4090)\n  `PR #4090 <https://github.com/pantsbuild/pants/pull/4090>`_\n\nDocumentation\n~~~~~~~~~~~~~\n\n* Remove stale example from 3rdparty_jvm.md (#4112)\n  `PR #4112 <https://github.com/pantsbuild/pants/pull/4112>`_\n\n* Add "common tasks" docs (#4060)\n  `PR #4060 <https://github.com/pantsbuild/pants/pull/4060>`_\n\n* Fix typo in docs (#4097)\n  `PR #4097 <https://github.com/pantsbuild/pants/pull/4097>`_\n\n1.3.0dev2 (11/20/2016)\n----------------------\n\nA return to the regular schedule of weekly unstable releases.\n\nAPI Changes\n~~~~~~~~~~~\n* Move SimpleCodegenTask into the pants core.\n  `PR #4079 <https://github.com/pantsbuild/pants/pull/4079>`_\n\n* Move the pytest-related runtime requirement specs  into a subsystem.\n  `PR #4071 <https://github.com/pantsbuild/pants/pull/4071>`_\n\n* Add the scala 2.12 platform\n  `RB #4388 <https://rbcommons.com/s/twitter/r/4388>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n* Fixup OSX bintray prep. (#4086)\n  `PR #4086 <https://github.com/pantsbuild/pants/pull/4086>`_\n\n* Task to gather local python sources into a pex.\n  `PR #4084 <https://github.com/pantsbuild/pants/pull/4084>`_\n\n* [engine] Initial Trace implementation for rust engine (#4076)\n  `Issue #4025 <https://github.com/pantsbuild/pants/issues/4025>`_\n  `PR #4076 <https://github.com/pantsbuild/pants/pull/4076>`_\n\n* Propose a github review workflow\n  `RB #4333 <https://rbcommons.com/s/twitter/r/4333>`_\n\n* Spelling mistake in first_tutorial (#4045)\n  `PR #4045 <https://github.com/pantsbuild/pants/pull/4045>`_\n\n* Replace instances of pantsbuild.github.io in the docs with pantsbuild.org.\n  `PR #4074 <https://github.com/pantsbuild/pants/pull/4074>`_\n\n* A task to resolve python requirements.\n  `PR #4065 <https://github.com/pantsbuild/pants/pull/4065>`_\n\n* Upgrade zinc\'s sbt dependency to 1.0.0: python portion\n  `RB #4064 <https://rbcommons.com/s/twitter/r/4064>`_\n  `RB #4340 <https://rbcommons.com/s/twitter/r/4340>`_\n  `RB #4342 <https://rbcommons.com/s/twitter/r/4342>`_\n\n* Skip failing tests to get CI green.\n  `RB #4391 <https://rbcommons.com/s/twitter/r/4391>`_\n\n* Avoid using expensive bootstrap artifacts from temporary cache location\n  `RB #4342 <https://rbcommons.com/s/twitter/r/4342>`_\n  `RB #4368 <https://rbcommons.com/s/twitter/r/4368>`_\n\n1.3.0dev1 (11/16/2016)\n----------------------\n\nThere has been a month gap between master releases and a corresponding large number of\nchanges. Most notably:\n\n* Pants now ships with a new native engine core that is the future of pants scalability work.\n\n* Pants has adopted a `code of conduct\n  <https://github.com/pantsbuild/pants/blob/master/CODE_OF_CONDUCT.md>`_\n\nAPI Changes\n~~~~~~~~~~~\n\n* Make findbugs task not transitive by default and modify findbugs progress output\n  `RB #4376 <https://rbcommons.com/s/twitter/r/4376>`_\n\n* Adding a Code of Conduct\n  `RB #4354 <https://rbcommons.com/s/twitter/r/4354>`_\n\n* Surface --dereference-symlinks flag to task caching level\n  `RB #4338 <https://rbcommons.com/s/twitter/r/4338>`_\n\n* support mutually_exclusive_group paramater in option registration\n  `RB #4336 <https://rbcommons.com/s/twitter/r/4336>`_\n\n* Deprecate the `java_tests` alias in favor of `junit_tests`.\n  `RB #4322 <https://rbcommons.com/s/twitter/r/4322>`_\n\n* Add a target-types option to scalafmt to avoid formatting all targets\n  `RB #4328 <https://rbcommons.com/s/twitter/r/4328>`_\n\n* Adding scalafmt formatting to fmt goal\n  `RB #4312 <https://rbcommons.com/s/twitter/r/4312>`_\n\nBugfixes\n~~~~~~~~\n\n* Capture testcase for unknown test failures in the JUnit Xml\n  `RB #4377 <https://rbcommons.com/s/twitter/r/4377>`_\n\n* Correction on [resolve.node]\n  `RB #4362 <https://rbcommons.com/s/twitter/r/4362>`_\n  `RB #4364 <https://rbcommons.com/s/twitter/r/4364>`_\n\n* Remove safe_mkdir on results_dir in [resolve.node]\n  `RB #4362 <https://rbcommons.com/s/twitter/r/4362>`_\n\n* Improve python_binary target fingerprinting.\n  `RB #4353 <https://rbcommons.com/s/twitter/r/4353>`_\n\n* Bugfix: when synthesizing remote libraries in Go, pin them to the same rev as adjacent libs.\n  `RB #4325 <https://rbcommons.com/s/twitter/r/4325>`_\n\n* Fix the SetupPy target ownership check.\n  `RB #4315 <https://rbcommons.com/s/twitter/r/4315>`_\n\n* Update junit runner to 1.0.15 to get java 7 compatibility\n  `RB #4324 <https://rbcommons.com/s/twitter/r/4324>`_\n\n* Fix erroneous deprecated scope warnings.\n  `RB #4323 <https://rbcommons.com/s/twitter/r/4323>`_\n\n* Back down the minimum required java version for running Pants tools to java 7\n  `RB #4127 <https://rbcommons.com/s/twitter/r/4127>`_\n  `RB #4253 <https://rbcommons.com/s/twitter/r/4253>`_\n  `RB #4314 <https://rbcommons.com/s/twitter/r/4314>`_\n\n* Fix exlucde_target_regexp breakage in test-changed and --files option breakage in changed with diffspec\n  `RB #4321 <https://rbcommons.com/s/twitter/r/4321>`_\n\n* Prevent cleanup error at end of pants test with --test-junit-html-report option, update safe_rmtree to be symlink aware\n  `RB #4319 <https://rbcommons.com/s/twitter/r/4319>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Format / Sort COMMITTERS.md; Add Yujie Chen to Active list\n  `RB #4382 <https://rbcommons.com/s/twitter/r/4382>`_\n\n* Bump junit-runner to 1.0.16\n  `RB #4381 <https://rbcommons.com/s/twitter/r/4381>`_\n\n* Patch to make scala tests work\n  `RB #4361 <https://rbcommons.com/s/twitter/r/4361>`_\n\n* Kill un-used `pants.jenkins.ini`.\n  `RB #4369 <https://rbcommons.com/s/twitter/r/4369>`_\n\n* Kill unused Jenkins experiment.\n  `RB #4366 <https://rbcommons.com/s/twitter/r/4366>`_\n\n* Split test_zinc_compile_integration into two smaller tests\n  `RB #4365 <https://rbcommons.com/s/twitter/r/4365>`_\n\n* Upgrade zinc\'s sbt dependency to 1.0.0: JVM portion\n  `Issue #144 <https://github.com/sbt/zinc/issues/144>`_\n  `Issue #151 <https://github.com/sbt/zinc/issues/151>`_\n  `Issue #185 <https://github.com/sbt/zinc/issues/185>`_\n  `RB #3658 <https://rbcommons.com/s/twitter/r/3658>`_\n  `RB #4342 <https://rbcommons.com/s/twitter/r/4342>`_\n  `RB #4340 <https://rbcommons.com/s/twitter/r/4340>`_\n\n* Perf improvement: rebase analyis file once instead of multiple times\n  `Issue #8 <https://github.com/pantsbuild/zincutils/issues/8>`_\n  `RB #4352 <https://rbcommons.com/s/twitter/r/4352>`_\n\n* Leverage default sources where possible.\n  `RB #4358 <https://rbcommons.com/s/twitter/r/4358>`_\n\n* [python-ng] A task to select a python interpreter.\n  `RB #4346 <https://rbcommons.com/s/twitter/r/4346>`_\n\n* Parallize thrift linter\n  `RB #4351 <https://rbcommons.com/s/twitter/r/4351>`_\n\n* normalize filespec exclude usage\n  `RB #4348 <https://rbcommons.com/s/twitter/r/4348>`_\n\n* clean up deprecated global_subsystems and task_subsystems\n  `RB #4349 <https://rbcommons.com/s/twitter/r/4349>`_\n\n* [jvm-compile] Ensure all invalid dependencies of targets are correctly represented in compile graph\n  `RB #4136 <https://rbcommons.com/s/twitter/r/4136>`_\n  `RB #4343 <https://rbcommons.com/s/twitter/r/4343>`_\n\n* Change default ./pants fmt.isort <empty> behavior to no-op; Add sources check for isort.\n  `RB #4327 <https://rbcommons.com/s/twitter/r/4327>`_\n\n* Allow targets to have sensible defaults for sources=.\n  `RB #4300 <https://rbcommons.com/s/twitter/r/4300>`_\n\n* Remove the long-deprecated Target.is_codegen().\n  `RB #4318 <https://rbcommons.com/s/twitter/r/4318>`_\n\n* Add one more shard to travis ci\n  `RB #4320 <https://rbcommons.com/s/twitter/r/4320>`_\n\nNew Engine Work\n~~~~~~~~~~~~~~~\n\n* Revert "Revert "Generate 32 bit native engine binaries.""\n  `RB #4380 <https://rbcommons.com/s/twitter/r/4380>`_\n  `Issue #4035 <https://github.com/pantsbuild/pants/issues/4035>`_\n\n* Add contrib, 3rdparty to copy list for mock buildroot as v2 engine to pass prefix checks.\n  `RB #4379 <https://rbcommons.com/s/twitter/r/4379>`_\n\n* Generate 32 bit native engine binaries.\n  `RB #4373 <https://rbcommons.com/s/twitter/r/4373>`_\n\n* Add support for publishing for OSX 10.7+.\n  `RB #4371 <https://rbcommons.com/s/twitter/r/4371>`_\n\n* Wire up native binary deploy to bintray.\n  `RB #4370 <https://rbcommons.com/s/twitter/r/4370>`_\n\n* Re-work native engine version.\n  `RB #4367 <https://rbcommons.com/s/twitter/r/4367>`_\n\n* First round of native engine feedback\n  `Issue #4020 <https://github.com/pantsbuild/pants/issues/4020>`_\n  `RB #4270 <https://rbcommons.com/s/twitter/r/4270>`_\n  `RB #4359 <https://rbcommons.com/s/twitter/r/4359>`_\n\n* [engine] Native scheduler implementation\n  `RB #4270 <https://rbcommons.com/s/twitter/r/4270>`_\n\n* Bootstrap the native engine from live sources.\n  `RB #4345 <https://rbcommons.com/s/twitter/r/4345>`_\n\n1.3.0dev0 (10/14/2016)\n----------------------\n\nThe first unstable release of the 1.3.x series.\n\nAPI Changes\n~~~~~~~~~~~\n\n* Add subsystem_utils to test_infra\n  `RB #4303 <https://rbcommons.com/s/twitter/r/4303>`_\n\nBugfixes\n~~~~~~~~\n\n* Switch default deference back to True for tarball artifact\n  `RB #4304 <https://rbcommons.com/s/twitter/r/4304>`_\n\n* Filter inactive goals from `Goal.all`.\n  `RB #4298 <https://rbcommons.com/s/twitter/r/4298>`_\n\n* JUnit runner fix for len(args) > max_args in argfile.safe_args\n  `RB #4294 <https://rbcommons.com/s/twitter/r/4294>`_\n\n* Fix --changed-files option\n  `RB #4309 <https://rbcommons.com/s/twitter/r/4309>`_\n\nRefactoring, Improvements, and Tooling\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n* Migrate changed integration tests to isolated temp git repos and add an environment variable to override buildroot\n  `RB #4295 <https://rbcommons.com/s/twitter/r/4295>`_\n\n* Get rid of the "Skipped X files" messages from isort output.\n  `RB #4301 <https://rbcommons.com/s/twitter/r/4301>`_\n\n* Version clarification\n  `RB #4299 <https://rbcommons.com/s/twitter/r/4299>`_\n\n* Fix isort to run `./pants fmt.isort` once.\n  `RB #4297 <https://rbcommons.com/s/twitter/r/4297>`_\n\n* Dogfood `./pants fmt.isort`.\n  `RB #4289 <https://rbcommons.com/s/twitter/r/4289>`_\n\n* Extract the junit xml report parser.\n  `RB #4292 <https://rbcommons.com/s/twitter/r/4292>`_\n\n* Leverage default targets throughout pants BUILDs.\n  `RB #4287 <https://rbcommons.com/s/twitter/r/4287>`_\n\nNew Engine Work\n~~~~~~~~~~~~~~~\n\n* [engine] Improve daemon run profiling.\n  `RB #4293 <https://rbcommons.com/s/twitter/r/4293>`_\n\n',
    'name': 'pantsbuild.pants.contrib.scrooge',
    'namespace_packages': ['pants', 'pants.contrib'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   'pants',
                    'pants.contrib',
                    'pants.contrib.scrooge',
                    'pants.contrib.scrooge.tasks'],
    'url': 'https://github.com/pantsbuild/pants',
    'version': '1.3.0',
    'zip_safe': True}
)



# coding=utf-8
# Copyright 2015 Pants project contributors (see CONTRIBUTORS.md).
# Licensed under the Apache License, Version 2.0 (see LICENSE).

from __future__ import (absolute_import, division, generators, nested_scopes, print_function,
                        unicode_literals, with_statement)

import hashlib

from pants.backend.codegen.thrift.java.java_thrift_library import JavaThriftLibrary
from pants.base.fingerprint_strategy import FingerprintStrategy


class JavaThriftLibraryFingerprintStrategy(FingerprintStrategy):
  """Scrooge cares about a Thrift target's `language` and `rpc_style` in addition to its payload.

  As such this strategy ensures new code is generated by Scrooge whenever any option that effects
  codegen changes.
  """

  def __init__(self, thrift_defaults):
    self._thrift_defaults = thrift_defaults

  def compute_fingerprint(self, target):
    fp = target.payload.fingerprint()
    if not isinstance(target, JavaThriftLibrary):
      return fp

    hasher = hashlib.sha1()
    hasher.update(fp)
    hasher.update(self._thrift_defaults.language(target))
    hasher.update(self._thrift_defaults.rpc_style(target))

    namespace_map = self._thrift_defaults.namespace_map(target)
    if namespace_map:
      hasher.update(str(sorted(namespace_map.items())))

    default_java_namespace = self._thrift_defaults.default_java_namespace(target)
    if default_java_namespace:
      hasher.update(default_java_namespace)

    if target.include_paths:
      hasher.update(str(target.include_paths))

    return hasher.hexdigest()

  def __hash__(self):
    return hash((type(self), self._thrift_defaults))

  def __eq__(self, other):
    return type(self) == type(other) and self._thrift_defaults == other._thrift_defaults
